@@ -297,7 +297,9 @@ class PeriodicSite(Site):
         """
         if self.lattice != other.lattice:
             return False
-        return  (abs(self._fcoords - other._fcoords) % 1 < tolerance).all()
+        frac_diff = abs(self._fcoords - other._fcoords) % 1
+        frac_diff = [abs(a) < tolerance or abs(a) > 1 - tolerance for a in frac_diff]
+        return  all(frac_diff)
     
     def __eq__(self, other):
         return self._species == other._species and self._lattice == other._lattice and np.allclose(self._coords, other._coords)
@@ -647,7 +649,7 @@ class Structure(collections.Sequence, collections.Hashable):
         """
         return [(s, dist) for (s, dist) in self.get_sites_in_sphere(site.coords, r) if site != s]
     
-    def get_all_neighbors(self, r):
+    def get_all_neighbors(self, r, include_index = False):
         """
         Get neighbors for each atom in the unit cell, out to a distance r
         Returns a list of list of neighbors for each site in structure.
@@ -663,9 +665,15 @@ class Structure(collections.Sequence, collections.Hashable):
         Arguments:
             r:
                 radius of sphere.
+            
+            include_index:
+                boolean that determines whether the non-supercell site index is included in the returned data
         
         Returns:
-            A list of a list of nearest neighbors for each site, i.e., [[(site, dist) ...], ..] 
+            A list of a list of nearest neighbors for each site, i.e., [[(site, dist, *index) ...], ..] 
+            *index only supplied if include_index = true
+            The index is the index of the site in the original (non-supercell) structure. This is needed for ewaldmatrix
+            by keeping track of which sites contribute to the ewald sum
         
         """
     
@@ -700,7 +708,9 @@ class Structure(collections.Sequence, collections.Hashable):
         site_coords = np.array(self.cart_coords)
         n = len(self._sites)
         for image in itertools.product(xrange(nxmin, nxmax+1), xrange(nymin, nymax+1), xrange(nzmin, nzmax+1)):
-            for site in unit_cell_sites:
+            for j in range(len(unit_cell_sites)):
+            #for site in unit_cell_sites:
+                site = unit_cell_sites[j]
                 fcoords = site.frac_coords + np.array(image)
                 coords = self.lattice.get_cartesian_coords(fcoords)
                 submat = [coords] * n
@@ -708,7 +718,9 @@ class Structure(collections.Sequence, collections.Hashable):
                 dists = np.sqrt(dists.sum(axis = 1))
                 withindists = (dists <= r) * (dists > 1e-8)
                 for i in xrange(n):
-                    if withindists[i]:
+                    if include_index & withindists[i]:
+                        neighbors[i].append((PeriodicSite(site.species_and_occu,fcoords, site.lattice), dists[i], j))
+                    elif withindists[i]:
                         neighbors[i].append((PeriodicSite(site.species_and_occu,fcoords, site.lattice), dists[i]))
                 
         return neighbors
@@ -794,7 +806,7 @@ class Structure(collections.Sequence, collections.Hashable):
         end_coords = np.array(end_structure.frac_coords)
 
         jimage=-np.array(np.around(end_coords-start_coords),int)
-        vec = end_coords - start_coords + jimage
+        vec = end_coords - start_coords #+ jimage
         intStructs = [Structure(self.lattice,[site.species_and_occu for site in self._sites],start_coords + float(x)/float(nimages) * vec) for x in xrange(0,nimages+1)]
         return intStructs;
     
@@ -916,8 +928,9 @@ class Composition (collections.Mapping, collections.Hashable):
         """
         if any([e < 0 for e in elmap.values()]):
             raise ValueError("Amounts in Composition cannot be negative!")
-        if not any([isinstance(e, (Element,Specie)) for e in elmap.keys()]):
-            raise TypeError("Keys must be instances of Element or Specie!")
+        for e in elmap.keys():
+            if not isinstance(e, (Element,Specie)):
+                raise TypeError("Keys must be instances of Element or Specie!")
         self._elmap = elmap.copy()
         self._natoms = sum(self._elmap.values())
 
@@ -1195,4 +1208,5 @@ class Composition (collections.Mapping, collections.Hashable):
 
 if __name__ == "__main__":
     import doctest
-    doctest.testmod()
\ No newline at end of file
+    doctest.testmod()
+    
\ No newline at end of file
