@@ -16,7 +16,7 @@ __date__ ="$Sep 23, 2011M$"
 
 import numpy as np
 from pymatgen.core.structure import Composition
-from pymatgen.phasediagram.pdmaker import GrandPotentialPhaseDiagram
+from pymatgen.phasediagram.pdmaker import PhaseDiagram, GrandPotentialPhaseDiagram
 from pymatgen.analysis.reaction_calculator import Reaction
 
 class PDAnalyzer(object):
@@ -56,12 +56,15 @@ class PDAnalyzer(object):
         Note that in reality, the test needs to provide a tolerance (set to 1e-8 by default) for numerical errors.
         """
         dim = len(self._pd.elements)
-        origin = np.array(self._pd.qhull_data[facet[0]][0:dim-1])
-        cm = np.array([np.array(self._pd.qhull_data[facet[i]][0:dim-1]) - origin for i in xrange(1, len(facet))])
-        row = [comp.get_atomic_fraction(self._pd.elements[i]) for i in xrange(1,len(self._pd.elements))]
-        compm = np.array(row) - origin
-        coeffs = np.linalg.solve(cm.transpose(), compm)
-        return (coeffs >= -PDAnalyzer.numerical_tol).all() and sum(coeffs) <= (1 + PDAnalyzer.numerical_tol)
+        if dim > 1:
+            origin = np.array(self._pd.qhull_data[facet[0]][0:dim-1])
+            cm = np.array([np.array(self._pd.qhull_data[facet[i]][0:dim-1]) - origin for i in xrange(1, len(facet))])
+            row = [comp.get_atomic_fraction(self._pd.elements[i]) for i in xrange(1,len(self._pd.elements))]
+            compm = np.array(row) - origin
+            coeffs = np.linalg.solve(cm.transpose(), compm)
+            return (coeffs >= -PDAnalyzer.numerical_tol).all() and sum(coeffs) <= (1 + PDAnalyzer.numerical_tol)
+        else:
+            return True
 
     def _get_facets(self,comp):
         """
@@ -72,8 +75,17 @@ class PDAnalyzer(object):
             if self._in_facet(facet, comp):
                 memberfacets.append(facet)
         return memberfacets
+    
+    def _get_facet(self,comp):
+        """
+        Get the facets that a composition falls into.
+        """
+        for facet in self._pd.facets:
+            if self._in_facet(facet, comp):
+                return facet
+        raise RuntimeError("No facet found for comp = {}".format(comp))
 
-    def get_decomposition(self,comp):
+    def get_decomposition(self, comp):
         """
         Provides the decomposition at a particular composition
         Arguments:
@@ -81,8 +93,7 @@ class PDAnalyzer(object):
         Returns:
             Decomposition as a dict of {PDEntry: amount}
         """
-        memberfacets = self._get_facets(comp)
-        facet = memberfacets[0]
+        facet = self._get_facet(comp)
         complist = [self._pd.qhull_entries[i].composition for i in facet]
         m = self._make_comp_matrix(complist)
         compm = self._make_comp_matrix([comp])
@@ -93,8 +104,24 @@ class PDAnalyzer(object):
             if abs(decompamts[i][0]) > PDAnalyzer.numerical_tol:
                 decomp[self._pd.qhull_entries[facet[i]]] = decompamts[i][0]
         return decomp
+    
+    def get_decomp_and_e_above_hull(self, entry):
+        """
+        Provides the decomposition and energy above convex hull for an entry
+        Arguments:
+            entry - A PDEntry like object
+        Returns:
+            (decomp, energy above convex hull)  Stable entries should have energy above hull of 0.
+        """
+        comp = entry.composition
+        eperatom = entry.energy_per_atom
+        decomp = self.get_decomposition(comp)
+        hullenergy = sum([entry.energy_per_atom*amt for entry, amt in decomp.items()])
+        if abs(eperatom) < PDAnalyzer.numerical_tol:
+            return (decomp, 0)
+        return (decomp, eperatom - hullenergy)
 
-    def get_e_above_hull(self,entry):
+    def get_e_above_hull(self, entry):
         """
         Provides the energy above convex hull for an entry
         Arguments:
@@ -102,13 +129,24 @@ class PDAnalyzer(object):
         Returns:
             Energy above convex hull of entry.  Stable entries should have energy above hull of 0.
         """
-        comp = entry.composition
-        eperatom = entry.energy_per_atom
-        decomp = self.get_decomposition(comp)
-        hullenergy = sum([entry.energy_per_atom*amt for entry, amt in decomp.items()])
-        if abs(eperatom) < PDAnalyzer.numerical_tol:
-            return 0
-        return eperatom - hullenergy
+        return self.get_decomp_and_e_above_hull(entry)[1]
+
+
+    def get_equilibrium_reaction_energy(self, entry):
+        """
+        Provides the reaction energy of a stable entry from the neighboring equilibrium stable entries.
+        (also known as the inverse distance to hull).
+        Arguments:
+            entry - A PDEntry like object
+        Returns:
+            Equilibrium reaction energy of entry.  Stable entries should have equilibrium reaction energy <= 0.
+        """
+        if entry not in self._pd.stable_entries:
+            raise ValueError("Equilibrium reaction energy is available only for stable entries.")
+        entries = [e for e in self._pd.all_entries if e != entry]
+        modpd = PhaseDiagram(entries, self._pd.elements)
+        analyzer = PDAnalyzer(modpd)
+        return analyzer.get_decomp_and_e_above_hull(entry)[1]
     
     def get_transition_chempots(self, element):
         """
@@ -152,21 +190,31 @@ class PDAnalyzer(object):
         """
         if element not in self._pd.elements:
             raise ValueError("get_transition_chempots can only be called with elements in the phase diagram.")
-        
         chempots = self.get_transition_chempots(element)
         stable_entries = self._pd.stable_entries
         gccomp = Composition({el:amt for el, amt in comp.items() if el != element})
-        
+        elref = self._pd.el_refs[element]
         elcomp = Composition.from_formula(element.symbol)
+        prev_decomp = [];
         evolution = []
+        def are_same_decomp(decomp1, decomp2):
+            for comp in decomp2:
+                if comp not in decomp1:
+                    return False
+            return True
+        
         for c in chempots:
             gcpd = GrandPotentialPhaseDiagram(stable_entries, {element:c-0.01}, self._pd.elements)
             analyzer = PDAnalyzer(gcpd)
             decomp = [gcentry.original_entry.composition for gcentry, amt in analyzer.get_decomposition(gccomp).items() if amt > 1e-5]
-            if elcomp not in decomp:
-                decomp.insert(0,elcomp)
-            rxn = Reaction([comp], decomp)
-            rxn.normalize_to(comp)
-            evolution.append({'chempot':c, 'evolution' : - rxn.coeffs[rxn.all_comp.index(elcomp)], 'reaction':rxn})
+            decomp_entries = [gcentry.original_entry for gcentry, amt in analyzer.get_decomposition(gccomp).items() if amt > 1e-5]
+
+            if not are_same_decomp(prev_decomp, decomp):
+                if elcomp not in decomp:
+                    decomp.insert(0,elcomp)
+                rxn = Reaction([comp], decomp)
+                rxn.normalize_to(comp)
+                prev_decomp = decomp
+                evolution.append({'chempot':c, 'evolution' : - rxn.coeffs[rxn.all_comp.index(elcomp)], 'element_reference': elref, 'reaction':rxn, 'entries':decomp_entries})
             
         return evolution
