@@ -13,18 +13,21 @@ __status__ = "Production"
 __date__ ="$Sep 23, 2011M$"
 
 import numpy as np
-import warnings
+import logging
 
+from pymatgen.command_line.qhull_caller import qconvex
 from entries import GrandPotPDEntry
 from scipy.spatial import Delaunay
 
+logger = logging.getLogger(__name__)
+
 class PhaseDiagram (object):
     '''
     Simple phase diagram class taking in elements and entries as inputs.
     '''
     FORMATION_ENERGY_TOLERANCE = 1e-11
     
-    def __init__(self, entries,elements = None):
+    def __init__(self, entries, elements = None):
         """
         Standard constructor for phase diagram.
         Arguments:
@@ -42,7 +45,6 @@ class PhaseDiagram (object):
         self._qhull_entries = None
         self._stable_entries = None
         self._all_entries_hulldata = None
-        self._el_refs = None
         self.make_phasediagram()
 
     @property
@@ -111,7 +113,6 @@ class PhaseDiagram (object):
         """
         return self._el_refs
 
-
     def get_form_energy(self,entry):
         '''
         Returns the formation energy for an entry (NOT normalized) from the
@@ -122,7 +123,7 @@ class PhaseDiagram (object):
             formation energy from the elementals references.
         '''
         comp = entry.composition
-        energy = entry.energy - sum([comp[el]*self.el_ref[el].energy_per_atom for el in comp.elements])
+        energy = entry.energy - sum([comp[el]*self._el_refs[el].energy_per_atom for el in comp.elements])
         return energy
         
     def get_form_energy_per_atom(self,entry):
@@ -137,7 +138,6 @@ class PhaseDiagram (object):
         data = list()
         for entry in entries_to_process:
             comp = entry.composition
-
             energy_per_atom = entry.energy_per_atom
             row = list()
             for i in xrange(1,len(self._elements)):
@@ -150,49 +150,51 @@ class PhaseDiagram (object):
         '''
         Make data suitable for convex hull procedure from the list of entries.
         '''
+        logger.debug("Creating convex hull data...")
         #Determine the elemental references based on lowest energy for each.
-        self.el_ref = dict()
+        self._el_refs = dict()
         for entry in self._all_entries:
             if entry.composition.is_element:
                 el = entry.composition.elements[0]
                 e_per_atom = entry.energy_per_atom
-                if el not in self.el_ref:
-                    self.el_ref[el] = entry
-                elif self.el_ref[el].energy_per_atom > e_per_atom:
-                    self.el_ref[el] = entry
+                if el not in self._el_refs:
+                    self._el_refs[el] = entry
+                elif self._el_refs[el].energy_per_atom > e_per_atom:
+                    self._el_refs[el] = entry
 
-        # Remove positive formation energy entries and duplicate entries
-        def in_list(entries_list, entry):
-            for test_entry in entries_list:
-                if abs(entry.energy - test_entry.energy) <= self.FORMATION_ENERGY_TOLERANCE and entry.composition == test_entry.composition:
-                    warnings.warn("Duplicate entry found!  Discarding...")
-                    return True
-            return False
-        
+        # Remove positive formation energy entries
         entries_to_process = list()
         for entry in self._all_entries:
-            if self.get_form_energy(entry) <= self.FORMATION_ENERGY_TOLERANCE:# and (not in_list(entries_to_process, entry)):
+            if self.get_form_energy(entry) <= self.FORMATION_ENERGY_TOLERANCE:
                 entries_to_process.append(entry)
+            else:
+                logger.debug("Removing positive formation energy entry {}".format(entry))
                 
         self._qhull_entries = entries_to_process
         return self._process_entries_qhulldata(entries_to_process)
 
-    def make_phasediagram(self):
+    def make_phasediagram(self, use_external_qconvex = False):
         stable_entries = set()
         dim = len(self._elements)
         self._qhull_data = self._create_convhull_data()
         if len(self._qhull_data) == dim:
             self._facets = [range(len(self._elements))]
         else:
-            self._facets = Delaunay(self._qhull_data).convex_hull
+            if not use_external_qconvex and dim < 5:
+                logger.debug("Computing hull using scipy.spatial.delaunay")
+                delau = Delaunay(self._qhull_data)
+                self._facets = delau.convex_hull
+            else:
+                logger.debug("> 4D hull encountered. Computing hull using external qconvex call.")
+                self._facets = qconvex(self._qhull_data)
+            logger.debug("Final facets are\n{}".format(self._facets))
+            
+            logger.debug("Removing vertical facets...")
             finalfacets = list()
-
-            # Remove vertical facets and elemental facets
             for facet in self._facets:
                 facetmatrix = np.zeros((len(facet),len(facet)))
                 count = 0
                 is_element_facet = True
-
                 for vertex in facet:
                     facetmatrix[count] = np.array(self._qhull_data[vertex])
                     facetmatrix[count, dim-1] = 1
@@ -201,6 +203,8 @@ class PhaseDiagram (object):
                         is_element_facet = False
                 if abs(np.linalg.det(facetmatrix)) > 1e-8 and (not is_element_facet):
                     finalfacets.append(facet)
+                else:
+                    logger.debug("Removing vertical facet : {}".format(facet))
             self._facets = finalfacets
 
         for facet in self._facets:
