@@ -16,6 +16,8 @@ __date__ = "Nov 27, 2011"
 import os
 import ConfigParser
 import itertools
+import math
+import subprocess
 
 import numpy as np
 import vtk
@@ -86,14 +88,7 @@ class StructureInteractorStyle(vtk.vtkInteractorStyleTrackballCamera):
         elif sym == 'r':
             parent.redraw(True)
         elif sym == "s":
-            w2i = vtk.vtkWindowToImageFilter()
-            writer = vtk.vtkPNGWriter()
-            w2i.SetInput(parent.ren_win)
-            w2i.Update()
-            writer.SetInputConnection(w2i.GetOutputPort())
-            writer.SetFileName("image.png")
-            parent.ren_win.Render()
-            writer.Write()
+            parent.write_image("image.png")
         elif sym == "Up":
             parent.rotate_view(1,90)
         elif sym == "Down":
@@ -180,6 +175,7 @@ class StructureVis(object):
         
         style = StructureInteractorStyle(self)
         self.iren.SetInteractorStyle(style)
+        
     
     def rotate_view(self, axis_ind = 0, angle = 0):
         """
@@ -194,22 +190,33 @@ class StructureVis(object):
             camera.Pitch(angle)
         self.ren_win.Render()
             
-    def save_image(self, filename = "image.png"):
+    def write_image(self, filename = "image.png", magnification = 1, image_format = "png"):
         """
-        Save render window to a png image.
+        Save render window to an image.
         
         Arguments:
             filename:
                 filename to save to. Defaults to image.png.
+            magnification:
+                magnification.  Use it to render high res images.
+            image_format:
+                choose between jpeg, png.  Png is the default.
         """
-        w2i = vtk.vtkWindowToImageFilter()
-        writer = vtk.vtkPNGWriter()
-        w2i.SetInput(self.ren_win)
-        w2i.Update()
-        writer.SetInputConnection(w2i.GetOutputPort())
+        render_large = vtk.vtkRenderLargeImage()
+        render_large.SetInput(self.ren)
+        if image_format == "jpeg":
+            writer = vtk.vtkJPEGWriter()
+            writer.SetQuality(80)
+        else:
+            writer = vtk.vtkPNGWriter()
+        
+        render_large.SetMagnification(magnification);
         writer.SetFileName(filename)
+
+        writer.SetInputConnection(render_large.GetOutputPort())
         self.ren_win.Render()
         writer.Write()
+        del render_large
     
     def redraw(self, reset_camera = False):
         """
@@ -264,7 +271,7 @@ class StructureVis(object):
             reset_camera:
                 Set to True to reset the camera to a default determined based on the structure.
         """
-        
+        self.ren.RemoveAllViewProps()
         m = SupercellMaker(structure, self.supercell)
         s = m.modified_structure
         
@@ -330,6 +337,14 @@ class StructureVis(object):
         self.structure = structure
         self.title = s.composition.formula
 
+    def zoom(self, factor):
+        """
+        Zoom the camera view by a factor.
+        """
+        camera = self.ren.GetActiveCamera()
+        camera.Zoom(factor)
+        self.ren_win.Render()
+        
     def show(self):
         """
         Display the visualizer
@@ -523,3 +538,35 @@ class StructureVis(object):
         picker.AddObserver("EndPickEvent", annotate_pick)
         self.picker = picker
         self.iren.SetPicker(picker)
+
+def make_movie(structures, output_filename = "movie.mp4", zoom = 1.0, fps = 20, bitrate = 10000, quality = 5):
+    """
+    Generate a movie from a sequence of structures using vtk and ffmpeg.
+    
+    Arguments:
+        structures:
+            sequence of structures
+        output_filename:
+            filename for structure output. defaults to movie.mp4
+        zoom:
+            A zoom to be applied to the visulizer. Defaults to 1.0
+        fps:
+            Frames per second for the movie. Defaults to 20.
+        bitrate:
+            Video bitate.  Defaults to 10000 (fairly high quality).
+        quality:
+            A quality scale. Defaults to 5.
+        
+    """
+    vis = StructureVis()
+    vis.show_help = False
+    vis.redraw()
+    vis.zoom(zoom)
+    sigfig = int(math.floor(math.log10(len(structures))) + 1)
+    filename = "image{0:0"+str(sigfig)+"d}.png"
+    for i, s in enumerate(structures):
+        vis.set_structure(s)
+        vis.write_image(filename.format(i), 3)
+    filename = "image%0"+str(sigfig)+"d.png"
+    args = ["ffmpeg", "-y", "-qscale", str(quality), "-r", str(fps), "-b", str(bitrate), "-i", filename, output_filename]
+    p = subprocess.Popen(args)
